from .connection import TaosRestConnection
from .cursor import TaosRestCursor
from .restclient import RestClient
from .errors import *

threadsafety = 0
paramstyle = "pyformat"


def connect(**kwargs) -> TaosRestConnection:
    """
   Keyword Arguments
   ----------------------------
   - host : str, optional.
       host to connect
   - user : str, optional.
       username used to log in
   - password : str, optional.
       password used to log in
    - port : int, optional.
       port to connect
   - timeout : int, optional.
       the optional timeout parameter specifies a timeout in seconds for blocking operations
   """
    return TaosRestConnection(**kwargs)



from sqlalchemy import types as sql_types
from sqlalchemy.engine import default, reflection


class TaosRestDialect(default.DefaultDialect):
    name = "taosrest"
    driver = "taosrest"
    supports_native_boolean = True
    implicit_returning = True

    def do_rollback(self, connection):
        pass

    def _get_server_version_info(self, connection):
        return tuple(connection.connection.server_info)

    @classmethod
    def dbapi(cls):
        import taosrest

        return taosrest

    def has_schema(self, connection, schema):
        return False

    def has_table(self, connection, table_name, schema=None):
        try:
            connection.cursor().execute(f"describe {table_name}")
            return True
        except:
            return False

    @reflection.cache
    def get_indexes(self, connection, table_name, schema=None, **kw):
        """
        Gets all indexes
        """
        # no index is supported by TDengine
        return []

    def get_columns(self, connection, table_name, schema=None, **kw):
        try:
            cursor = connection.cursor()
            cursor.execute("describe {}" % table_name)
            return [row[0] for row in cursor.fetchall()]
        except:
            return []
